// Craig S. Kaplan
// A somewhat complicated demonstration of using text input
// and output to implement "persistence".  The sketch implements
// an annoying games: click in ten random rectangles as quickly
// as possible.  The interesting part is that you can quick and
// restart the sketch, and it'll remember your best score.

// The x and y coordinates of the corners of the
// rectangle we want the user to click on.
float x1;
float y1;
float x2;
float y2;

// This game requires several "modes" -- fundamentally
// different screens we show to the user.  So we define
// a couple of constants to give those modes names, and
// then a global variable to remember what mode we're
// in right now.
int WAITING = 0;
int PLAYING = 1;
int SCORE = 2;

int mode = 0;

// How many rectangles have we successfully clicked on
// in this round?
int num_completed;
// What time did we start the round?
int start_time;
// Once we've finished the round, how long did it take us?
float total_time;
// What the overall high score across all runs?  We
// initialize this value to -1 to indicate that the
// value isn't valid yet.
float best_time = -1;

void setup()
{
  size( 500, 500 );
  textSize( 30 );
  textAlign( CENTER );

  // When the sketch first starts, implement *persistence*:
  // Check if there happens to be a saved high score file.
  // If there is, read it and extract the high score
  // (which is just the floating-point value represented
  // in text on the first line of the file).
  String[] lines = loadStrings( "highscore.txt" );
  if ( lines != null ) {
    best_time = float( lines[0] );
  }
}

void draw()
{
  background( 50 );

  // What we actually draw depends on what mode we're in.
  if ( mode == WAITING ) {
    fill( 255 );
    // If we're waiting for the user to start, we display
    // a message and wait for a mouse press.
    text( "CLICK TO BEGIN", width/2, height/2 );
  } else if ( mode == SCORE ) {
    fill( 255 );
    // If we're at the end of the round, we display the
    // user's time and the best time, and wait for them
    // to click.
    text( "YOUR TIME: " + total_time, width/2, height/2 );
    if ( best_time >= 0 ) {
      text( "BEST TIME: " + best_time, 
        width/2, height/2 + 40 );
    }
  } else {
    // Otherwise we have to do some work.  Generate a
    // random rectangle and dispay it.  This is kind of
    // bad design -- really, we should ensure that the
    // rectangle is never so thin that it's basically
    // impossible to click on it.
    x1 = random( width );
    x2 = random( width );
    if ( x1 > x2 ) {
      float tmp = x2;
      x2 = x1;
      x1 = tmp;
    }
    y1 = random( height );
    y2 = random( height );
    if ( y1 > y2 ) {
      float tmp = y2;
      y2 = y1;
      y1 = tmp;
    }

    colorMode( HSB, 10 );
    fill( 9 - num_completed, 10, 10 );
    colorMode( RGB, 255 );
    
    rect( x1, y1, x2 - x1, y2 - y1 );

    // Use our old friend, the noLoop() trick.  That
    // way, we can generate a single random rectangle
    // here, and wait for the user to click on it before
    // turning looping back on.
    noLoop();
  }
}

void mousePressed()
{
  // Respond to a mouse press.  Once again, the action
  // we take depends on what more we're in.
  if ( mode == WAITING || mode == SCORE ) {
    // If we're not currently playing the game, any
    // click should restart.
    mode = PLAYING;
    num_completed = 0;
    start_time = millis();
  } else if ( mode == PLAYING ) { 
    // If we are playing, we need to "hit test" the
    // click location against the rectangle to see if
    // the user was successful.
    if ( mouseX > x1 && mouseX < x2 ) {
      if ( mouseY > y1 && mouseY < y2 ) {
        ++num_completed;
        if ( num_completed == 10 ) {
          // If you're clicked on ten rectangles, 
          // you win!!
          mode = SCORE;
          // How long did it take you, in seconds?
          total_time = (millis() - start_time) / 1000.0;
          // Did you beat the current high score?
          if ( best_time < 0 || total_time < best_time ) {
            // If so, remember the new best score in the
            // global variable...
            best_time = total_time;
            
            // ...and also persistently, by writing the
            // score to the highscore.txt file.
            PrintWriter pw = createWriter( "highscore.txt" );
            pw.println( best_time );
            pw.flush();
            pw.close();
          }
        }
        
        // Either way, it's time to turn looping back on,
        // so that we can draw a new rectangle.
        loop();
      }
    }
  }
}